# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.1

import datetime
import sys
import types
import typing
import uuid as _uuid
from builtins import property as _property

import winrt._winrt
import winrt.system
import winrt.windows.graphics.directx

from winrt.windows.graphics.directx.direct3d11 import Direct3DBindings, Direct3DUsage

Self = typing.TypeVar('Self')

class Direct3DMultisampleDescription:
    count: winrt.system.Int32
    quality: winrt.system.Int32
    def __init__(self, count: winrt.system.Int32, quality: winrt.system.Int32) -> None: ...

class Direct3DSurfaceDescription:
    width: winrt.system.Int32
    height: winrt.system.Int32
    format: winrt.windows.graphics.directx.DirectXPixelFormat
    multisample_description: Direct3DMultisampleDescription
    def __init__(self, width: winrt.system.Int32, height: winrt.system.Int32, format: winrt.windows.graphics.directx.DirectXPixelFormat, multisample_description: Direct3DMultisampleDescription) -> None: ...

class IDirect3DDevice(winrt.system.Object):
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, *args) -> None: ...
    @staticmethod
    def _from(obj: winrt.system.Object, /) -> IDirect3DDevice: ...
    def close(self) -> None: ...
    def trim(self) -> None: ...

class IDirect3DSurface(winrt.system.Object):
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, *args) -> None: ...
    @staticmethod
    def _from(obj: winrt.system.Object, /) -> IDirect3DSurface: ...
    def close(self) -> None: ...
    @_property
    def description(self) -> Direct3DSurfaceDescription: ...

