# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.1

import enum

import winrt.system
from winrt import _winrt_windows_graphics_directx_direct3d11

__all__ = [
    "Direct3DBindings",
    "Direct3DUsage",
    "Direct3DMultisampleDescription",
    "Direct3DSurfaceDescription",
    "IDirect3DDevice",
    "IDirect3DSurface",
]

class Direct3DBindings(enum.IntFlag):
    VERTEX_BUFFER = 0x1
    INDEX_BUFFER = 0x2
    CONSTANT_BUFFER = 0x4
    SHADER_RESOURCE = 0x8
    STREAM_OUTPUT = 0x10
    RENDER_TARGET = 0x20
    DEPTH_STENCIL = 0x40
    UNORDERED_ACCESS = 0x80
    DECODER = 0x200
    VIDEO_ENCODER = 0x400

class Direct3DUsage(enum.IntEnum):
    DEFAULT = 0
    IMMUTABLE = 1
    DYNAMIC = 2
    STAGING = 3

Direct3DMultisampleDescription = _winrt_windows_graphics_directx_direct3d11.Direct3DMultisampleDescription
Direct3DSurfaceDescription = _winrt_windows_graphics_directx_direct3d11.Direct3DSurfaceDescription
IDirect3DDevice = _winrt_windows_graphics_directx_direct3d11.IDirect3DDevice
IDirect3DSurface = _winrt_windows_graphics_directx_direct3d11.IDirect3DSurface
